"""
Base class for all Resources that handle the base crud operations 
"""
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from moraqib._client import Moraqib, AsyncMoraqib


class Resource:
    _client: Moraqib

    def __init__(self, client:Moraqib) -> None:
        self._client = client 
    

class AsyncResource:
    _client: AsyncMoraqib 

    def __init__(self, client: AsyncMoraqib) -> None:
        self._client = client


from pydantic import BaseModel
from typing import TypeVar
from uuid import UUID

ID = TypeVar(UUID, int, str,)


class BaseAPIModel(BaseModel):

    def json():
        pass 

    def dict(self):
        pass 

    class Config:
        """TODO:: add more configurations """


# TODO:: see how to auto generate using openapi spces 
from typing import Optional
from moraqib.types.common import ID, BaseAPIModel
from pydantic import Field
import datetime

class Trace(BaseAPIModel):
    name: Optional[str]
    """The name of the trace"""
    latency: Optional[float]
    """The latency of the trace"""
    tota_cost: float 
    """cost of trace in USD """
    observations: list[ID]
    """The List of observations' ids of the trace"""
    scores: list[float]
    """The list of scores of the trace"""
    timestamp: datetime.datetime 
    """The timestamp of the trace"""    
    session_id: ID 
    """The session id of the trace"""
    user_id: ID 
    """The user id of the trace"""
    project_id: ID 
    """The project id of the trace"""
    is_public: Optional[bool] = Field(type=bool, default=False)
    """Whether the trace is public"""
    tags: list[str] = []
    """The list of tags of the trace"""


class CreateTrace(BaseAPIModel):
    name: Optional[str]
    used_id: Optional[str]
    session_id: Optional[str]
    tags: list[str] = []
    public: Optional[bool] = Field(type=bool, default=False)

# TODO:: add more traces CRUD class bodies 


from moraqib._exceptions import MoraqibError 
from moraqib.resources.trace import Trace
from typing import Optional, Mapping , Any
from typing import TypeVar
import httpx 
import os

Body = TypeVar("Body", Mapping[str, Any],None)


class Moraqib():
    """
    Moraqib is a Python class for managing a Moraqib's inventory.
    """

    trace: Trace
    def __init__(self,
                 base_url:Optional[str] | httpx.URL=None,
                 *,
                 api_key:str,
                 http_client: httpx.Client=None,
                 ) -> None:
        if not base_url:
            self.base_url = "http://localhost:5000/api/v1"
        else:
            self.base_url = base_url

        self.api_key = api_key
        if not self.api_key:
            self.api_key = os.environ.get('MORAQIB_API_KEY')
        
        if not self.api_key:
            raise MoraqibError("Moraqib API key is not provided. Please provide it as a parameter or set the environment variable 'MORAQIB_API_KEY'")
        
        self.http_client = http_client or httpx.Client(
            base_url=self.base_url,
            auth=(self.api_key, ''),
            headers=self._headers,
            # limits= ## TODO:: handle limits 
        )

        # Main Resources 
        self.trace = Trace(self)

    @property
    def _headers(self):
        return {
            # 'Authorization': f'Bearer {self.api_key}',
            'Content-Type': 'application/json'
        }


    def get(self, 
            path:str,
            *,
            stream:bool=False
            ):
        return self.http_client.get(path,headers=self._headers,stream=stream) # TODO:: handle custom  stream Response 

    def post(self,
            path:str,
            *,
            body: Optional[Body],
            stream:bool=False
            ):
        # TODO:: use generics to handle cast operation 
        return self.http_client.post(path,headers=self._headers,data=body,stream=stream) # TODO:: handle custom  stream Response 

    def put(self):
        pass 

    def patch(self):
        pass 

    def delete(self):
        pass 



class AsyncMoraqib():
    """
        TODO:: add support for async httpx
    """


from ._resource import Resource, AsyncResource
from moraqib._client import Moraqib

class Trace(Resource):
    """
    Trace is a resource that handles trace operations
    """
    endpoint = "/traces/"
    def __init__(self, client: Moraqib):
        super().__init__(client) 

        
    def create(self,*args,**kwargs):
        return self._client.post(
           self.endpoint,
           
        )

class AsyncTrace(AsyncResource):
    pass 